import sys

from datetime import datetime
 
from datetime import timedelta  
from time import sleep
from IOTAssignmentUtilitiesdorachua.MySQLManager import MySQLManager
from IOTAssignmentUtilitiesdorachua.MySQLManager import QUERYTYPE_DELETE,QUERYTYPE_RETRIEVE
from IOTAssignmentServerdorachua.GrabCarV2 import GrabCarV2
from IOTAssignmentServerdorachua.SocketFeedV2 import SocketFeedV2

class MyNewCarsFeederV2:

    def __init__(self,u,pw,h,db):
        self.user = u
        self.password = pw
        self.host = h
        self.database = db
    
    
    def getRandomBookingIDs(self,numbertoget):

        bids = []

        try:

            self.mysqlm =  MySQLManager(self.user, self.password,self.host,self.database)
            self.mysqlm.connect()

            result = self.mysqlm.retrieve(QUERYTYPE_RETRIEVE, f"SELECT bookingid FROM telemetry ORDER BY RAND() LIMIT {numbertoget}",{})

            if result == True:
                results = self.mysqlm.cursor.fetchall()

                for r in results:
                    bids.append(r[0])                
            else:
                print(f"Error retrieving {numbertoget} random booking ids")

        except:
            print(sys.exc_info()[0])
            print(sys.exc_info()[1])

        finally:
            self.mysqlm.disconnect()

        return bids
    
    def createCars(self,numbertoget):

        try:
            cars = []

            datetime_start = datetime.now()

            bids = self.getRandomBookingIDs(numbertoget)
            
            for bid in bids:
                car = GrabCarV2(datetime.now(),bid,self.user, self.password,self.host,self.database)                
                car.recordIntoDB()
                cars.append(car)            
        
            return cars

        except:
            print(sys.exc_info()[0])
            print(sys.exc_info()[1])
            return cars

    def getAllCars(self):

        try:
            socketfeedv2 = []

            self.mysqlm =  MySQLManager(self.user, self.password,self.host,self.database)
            self.mysqlm.connect()

            sql = f"SELECT * FROM socketfeedv2"
            sqldata = {}
            result = self.mysqlm.retrieve(QUERYTYPE_RETRIEVE,sql,sqldata)
            if result == True:
                results = self.mysqlm.cursor.fetchall()

            for r in results:
                sfv2 = SocketFeedV2(r,self.user, self.password,self.host,self.database)
                socketfeedv2.append(sfv2)

            self.mysqlm.disconnect()

            return socketfeedv2

        except:
            print(sys.exc_info()[0])
            print(sys.exc_info()[1])
            return socketfeedv2


    def truncateDB(self):

        try:                    
            
            self.mysqlm =  MySQLManager(self.user, self.password,self.host,self.database)
            self.mysqlm.connect()

            print("Truncating records from database first...")
            self.mysqlm.insertupdatedelete(QUERYTYPE_DELETE, "DELETE FROM socketfeedv2",{})

            self.mysqlm.disconnect()

        except KeyboardInterrupt:
            print('Interrupted')
            sys.exit()

        except:
            print(sys.exc_info()[0])
            print(sys.exc_info()[1])


import asyncio
import sys
from random import randint,random
from datetime import datetime, timedelta 

import json
from IOTAssignmentUtilitiesdorachua import MySQLManager
from IOTAssignmentServerdorachua.GrabCarV2 import GrabCarV2
from IOTAssignmentServerdorachua.MyNewCarsFeederV2 import MyNewCarsFeederV2
import argparse

class MySocketServer:

    def __init__(self,u,pw,h,db):
        self.user = u
        self.password = pw
        self.host = h
        self.database = db
        self.isconnected = False
        self.feeder = None
        self.lastfeedtime = datetime.now()
        self.updateinterval = 2
        self.nextfeedtime = self.laststarttime + timedelta(minutes = self.updateinterval)

    def setNewCarsFeeder(self,feeder):
        self.feeder = feeder

    async def handle_client(self,reader, writer, cars, timeout,nextfeedtime,updateinterval):

        try:            
            self.updateinterval = updateinterval            
            self.nextfeedtime = self.laststarttime + timedelta(minutes = self.updateinterval)
            timenow = datetime.now()
                        
            if timenow >= self.nextfeedtime and self.feeder is not None:
                self.lastfeedtime = self.nextfeedtime
                self.nextfeedtime = self.laststarttime + timedelta(minutes = self.updateinterval)
                newcars = self.feeder.createCars(5)
                
            data = await reader.read(100)
            message = data.decode("utf-8")
            requested_time = int(message)            
            addr = writer.get_extra_info('peername')

            dt = datetime.now()
            fn = f"{dt.year}-{dt.month}-{dt.day} {dt.hour}:{dt.minute}:{dt.second}"
            
            print(f"Received {message} from {addr} at {fn}")        
            
            str_to_send = ""
            readings = []
            
            for i in range(0,len(cars)):                
                reading = cars[i].read(requested_time-cars[i].offsetseconds)
                if reading is not None:
                    readings.append(reading)
                
            if len(readings)>0:            
                writer.write(json.dumps(readings).encode("utf-8"))
                await writer.drain()            
            
                #print("Close the connection")

            writer.close()    
        
        except TypeError as te:
            print("Encountering TypeError in handle_client")
            for e in sys.exc_info():
                print(e)

        except ValueError as ve:
            print("Encountering ValueError in handle_client")
            for e in sys.exc_info():
                print(e)
        
        except KeyboardInterrupt:
            print('Interrupted')
            sys.exit()

        except:
            print("Error occurred in handle_client")
            print(sys.exc_info()[0])
            print(sys.exc_info()[1])






from time import sleep
from IOTAssignmentUtilitiesdorachua.MySQLManager import MySQLManager
from IOTAssignmentUtilitiesdorachua.MySQLManager import QUERYTYPE_DELETE,QUERYTYPE_INSERT,QUERYTYPE_RETRIEVE,QUERYTYPE_UPDATE
from IOTAssignmentUtilitiesdorachua.OurCustomEncoder import OurCustomEncoder
import json
import sys
from datetime import datetime,timedelta
#from IOTAssignmentUtilitiesdorachua.MyDateTimeUtilities import MyDateTimeUtilities

class GrabCarV2:
  
    def __init__(self,appstarttime,bid, u,pw,h,db):        
        self.user = u
        self.password = pw
        self.host = h
        self.database = db
        self.bookingid = bid
        self.appstarttime = appstarttime
        #self.recordIntoDB()    

    def getDuration(self):

        try:
                
            self.mysqlm =  MySQLManager(self.user, self.password,self.host,self.database)
            self.mysqlm.connect()

            sql = f"SELECT MAX(seconds) as lastrecordedsecond FROM telemetry WHERE bookingid=%(bookingid)s"
            sqldata = {"bookingid": self.bookingid}

            result = self.mysqlm.retrieve(QUERYTYPE_RETRIEVE, sql, sqldata)

            if result == True:
                record = self.mysqlm.cursor.fetchone()
                duration = record[0]
                self.mysqlm.disconnect()
                return True,duration

            else:
                print(f"Error retrieving the data of booking id {self.bookingid}")
                print(sys.exc_info()[0])
                print(sys.exc_info()[1])
                
                return False,-1
        except TypeError:
            print("TypeError in getDuration")

        finally:
            print("Exited getDuration")
            self.mysqlm.disconnect()

    def recordIntoDB(self):

        try:
            
            getDuration_isSuccessful, duration = self.getDuration()

            self.startdatetime_value = datetime.now()
            self.enddatetime_value = self.startdatetime_value + timedelta(seconds = duration)
            self.bookingidwithtime = f"{self.bookingid}_{str(self.appstarttime)}"

            #mdtu = MyDateTimeUtilities()        
            #self.offsetseconds = mdtu.date_diff_in_seconds(self.startdatetime_value-self.appstarttime)
            timedelta_value = self.startdatetime_value-self.appstarttime
            self.offsetseconds = timedelta_value.days * 24 * 3600 + timedelta_value.seconds

            sql = "INSERT INTO socketfeedv2 (bookingid,bookingidwithtime,offsetseconds,startdatetime_value,enddatetime_value) VALUES (%(bookingid)s,%(bookingidwithtime)s,%(offsetseconds)s,%(startdatetime_value)s,%(enddatetime_value)s)"
            sqldata = {'bookingid': self.bookingid, 'bookingidwithtime': self.bookingidwithtime, 'offsetseconds': self.offsetseconds,
                    'startdatetime_value': self.startdatetime_value, 'enddatetime_value':self.enddatetime_value}

            self.mysqlm =  MySQLManager(self.user, self.password,self.host,self.database)
            self.mysqlm.connect()

            insert_isSuccessful = self.mysqlm.insertupdatedelete(QUERYTYPE_INSERT,sql,sqldata)
            if insert_isSuccessful:
                print(f"{sqldata} inserted")
                
        except TypeError:
            print("TypeError in recordIntoDB")
        finally:
            self.mysqlm.disconnect()



    def getBookingID(self):
        return self.bookingid 




from time import sleep
from IOTAssignmentUtilitiesdorachua.MySQLManager import MySQLManager
from IOTAssignmentUtilitiesdorachua.MySQLManager import QUERYTYPE_DELETE,QUERYTYPE_INSERT,QUERYTYPE_RETRIEVE,QUERYTYPE_UPDATE
from IOTAssignmentUtilitiesdorachua.OurCustomEncoder import OurCustomEncoder
import json
import sys
from datetime import datetime,timedelta
#from IOTAssignmentUtilitiesdorachua.MyDateTimeUtilities import MyDateTimeUtilities

class SocketFeedV2:
  
    def __init__(self,record, u,pw,h,db):        
        self.user = u
        self.password = pw
        self.host = h
        self.database = db

        print(record)

        self.id = record["id"]
        self.bookingid = record["bookingid"]
        self.bookingidwithtime = record['bookingidwithtime']
        self.offsetseconds = record['offsetsecords']
        self.startdatetime_value = record['startdatetime_value']
        self.enddatetime_value = record['enddatetime_value']
        self.timestamp_value = record['timestamp_value']

        

        
    